@@ -1,10 +1,11 @@
-""" Access and control log capturing. """
+"""Access and control log capturing."""
 import logging
 import os
 import re
 import sys
 from contextlib import contextmanager
 from io import StringIO
+from pathlib import Path
 from typing import AbstractSet
 from typing import Dict
 from typing import Generator
@@ -15,18 +16,21 @@ from typing import Tuple
 from typing import TypeVar
 from typing import Union
 
-import pytest
 from _pytest import nodes
 from _pytest._io import TerminalWriter
 from _pytest.capture import CaptureManager
+from _pytest.compat import final
 from _pytest.compat import nullcontext
 from _pytest.config import _strtobool
 from _pytest.config import Config
 from _pytest.config import create_terminal_writer
+from _pytest.config import hookimpl
+from _pytest.config import UsageError
 from _pytest.config.argparsing import Parser
+from _pytest.deprecated import check_ispytest
+from _pytest.fixtures import fixture
 from _pytest.fixtures import FixtureRequest
 from _pytest.main import Session
-from _pytest.pathlib import Path
 from _pytest.store import StoreKey
 from _pytest.terminal import TerminalReporter
 
@@ -43,11 +47,10 @@ def _remove_ansi_escape_sequences(text: str) -> str:
 
 
 class ColoredLevelFormatter(logging.Formatter):
-    """
-    Colorize the %(levelname)..s part of the log format passed to __init__.
-    """
+    """A logging formatter which colorizes the %(levelname)..s part of the
+    log format passed to __init__."""
 
-    LOGLEVEL_COLOROPTS = {
+    LOGLEVEL_COLOROPTS: Mapping[int, AbstractSet[str]] = {
         logging.CRITICAL: {"red"},
         logging.ERROR: {"red", "bold"},
         logging.WARNING: {"yellow"},
@@ -55,13 +58,13 @@ class ColoredLevelFormatter(logging.Formatter):
         logging.INFO: {"green"},
         logging.DEBUG: {"purple"},
         logging.NOTSET: set(),
-    }  # type: Mapping[int, AbstractSet[str]]
+    }
     LEVELNAME_FMT_REGEX = re.compile(r"%\(levelname\)([+-.]?\d*s)")
 
     def __init__(self, terminalwriter: TerminalWriter, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
         self._original_fmt = self._style._fmt
-        self._level_to_fmt_mapping = {}  # type: Dict[int, str]
+        self._level_to_fmt_mapping: Dict[int, str] = {}
 
         assert self._fmt is not None
         levelname_fmt_match = self.LEVELNAME_FMT_REGEX.search(self._fmt)
@@ -110,7 +113,7 @@ class PercentStyleMultiline(logging.PercentStyle):
 
     @staticmethod
     def _get_auto_indent(auto_indent_option: Union[int, str, bool, None]) -> int:
-        """Determines the current auto indentation setting
+        """Determine the current auto indentation setting.
 
         Specify auto indent behavior (on/off/fixed) by passing in
         extra={"auto_indent": [value]} to the call to logging.log() or
@@ -128,12 +131,14 @@ class PercentStyleMultiline(logging.PercentStyle):
         Any other values for the option are invalid, and will silently be
         converted to the default.
 
-        :param any auto_indent_option: User specified option for indentation
-            from command line, config or extra kwarg. Accepts int, bool or str.
-            str option accepts the same range of values as boolean config options,
-            as well as positive integers represented in str form.
+        :param None|bool|int|str auto_indent_option:
+            User specified option for indentation from command line, config
+            or extra kwarg. Accepts int, bool or str. str option accepts the
+            same range of values as boolean config options, as well as
+            positive integers represented in str form.
 
-        :returns: indentation value, which can be
+        :returns:
+            Indentation value, which can be
             -1 (automatically determine indentation) or
             0 (auto-indent turned off) or
             >0 (explicitly set indentation position).
@@ -164,7 +169,7 @@ class PercentStyleMultiline(logging.PercentStyle):
     def format(self, record: logging.LogRecord) -> str:
         if "\n" in record.message:
             if hasattr(record, "auto_indent"):
-                # passed in from the "extra={}" kwarg on the call to logging.log()
+                # Passed in from the "extra={}" kwarg on the call to logging.log().
                 auto_indent = self._get_auto_indent(record.auto_indent)  # type: ignore[attr-defined]
             else:
                 auto_indent = self._auto_indent
@@ -178,7 +183,7 @@ class PercentStyleMultiline(logging.PercentStyle):
                         lines[0]
                     )
                 else:
-                    # optimizes logging by allowing a fixed indentation
+                    # Optimizes logging by allowing a fixed indentation.
                     indentation = auto_indent
                 lines[0] = formatted
                 return ("\n" + " " * indentation).join(lines)
@@ -313,12 +318,12 @@ class catching_logs:
 class LogCaptureHandler(logging.StreamHandler):
     """A logging handler that stores log records and the log text."""
 
-    stream = None  # type: StringIO
+    stream: StringIO
 
     def __init__(self) -> None:
-        """Creates a new log handler."""
+        """Create a new log handler."""
         super().__init__(StringIO())
-        self.records = []  # type: List[logging.LogRecord]
+        self.records: List[logging.LogRecord] = []
 
     def emit(self, record: logging.LogRecord) -> None:
         """Keep the log records in a list in addition to the log text."""
@@ -338,22 +343,23 @@ class LogCaptureHandler(logging.StreamHandler):
             raise
 
 
+@final
 class LogCaptureFixture:
     """Provides access and control of log capturing."""
 
-    def __init__(self, item: nodes.Node) -> None:
-        """Creates a new funcarg."""
+    def __init__(self, item: nodes.Node, *, _ispytest: bool = False) -> None:
+        check_ispytest(_ispytest)
         self._item = item
-        # dict of log name -> log level
-        self._initial_handler_level = None  # type: Optional[int]
-        self._initial_logger_levels = {}  # type: Dict[Optional[str], int]
+        self._initial_handler_level: Optional[int] = None
+        # Dict of log name -> log level.
+        self._initial_logger_levels: Dict[Optional[str], int] = {}
 
     def _finalize(self) -> None:
-        """Finalizes the fixture.
+        """Finalize the fixture.
 
         This restores the log levels changed by :meth:`set_level`.
         """
-        # restore log levels
+        # Restore log levels.
         if self._initial_handler_level is not None:
             self.handler.setLevel(self._initial_handler_level)
         for logger_name, level in self._initial_logger_levels.items():
@@ -362,20 +368,20 @@ class LogCaptureFixture:
 
     @property
     def handler(self) -> LogCaptureHandler:
-        """
+        """Get the logging handler used by the fixture.
+
         :rtype: LogCaptureHandler
         """
         return self._item._store[caplog_handler_key]
 
     def get_records(self, when: str) -> List[logging.LogRecord]:
-        """
-        Get the logging records for one of the possible test phases.
+        """Get the logging records for one of the possible test phases.
 
         :param str when:
             Which test phase to obtain the records from. Valid values are: "setup", "call" and "teardown".
 
+        :returns: The list of captured records at the given stage.
         :rtype: List[logging.LogRecord]
-        :return: the list of captured records at the given stage
 
         .. versionadded:: 3.4
         """
@@ -383,17 +389,17 @@ class LogCaptureFixture:
 
     @property
     def text(self) -> str:
-        """Returns the formatted log text."""
+        """The formatted log text."""
         return _remove_ansi_escape_sequences(self.handler.stream.getvalue())
 
     @property
     def records(self) -> List[logging.LogRecord]:
-        """Returns the list of log records."""
+        """The list of log records."""
         return self.handler.records
 
     @property
     def record_tuples(self) -> List[Tuple[str, int, str]]:
-        """Returns a list of a stripped down version of log records intended
+        """A list of a stripped down version of log records intended
         for use in assertion comparison.
 
         The format of the tuple is:
@@ -404,15 +410,18 @@ class LogCaptureFixture:
 
     @property
     def messages(self) -> List[str]:
-        """Returns a list of format-interpolated log messages.
+        """A list of format-interpolated log messages.
 
-        Unlike 'records', which contains the format string and parameters for interpolation, log messages in this list
-        are all interpolated.
-        Unlike 'text', which contains the output from the handler, log messages in this list are unadorned with
-        levels, timestamps, etc, making exact comparisons more reliable.
+        Unlike 'records', which contains the format string and parameters for
+        interpolation, log messages in this list are all interpolated.
 
-        Note that traceback or stack info (from :func:`logging.exception` or the `exc_info` or `stack_info` arguments
-        to the logging functions) is not included, as this is added by the formatter in the handler.
+        Unlike 'text', which contains the output from the handler, log
+        messages in this list are unadorned with levels, timestamps, etc,
+        making exact comparisons more reliable.
+
+        Note that traceback or stack info (from :func:`logging.exception` or
+        the `exc_info` or `stack_info` arguments to the logging functions) is
+        not included, as this is added by the formatter in the handler.
 
         .. versionadded:: 3.7
         """
@@ -423,32 +432,33 @@ class LogCaptureFixture:
         self.handler.reset()
 
     def set_level(self, level: Union[int, str], logger: Optional[str] = None) -> None:
-        """Sets the level for capturing of logs. The level will be restored to its previous value at the end of
-        the test.
-
-        :param int level: the logger to level.
-        :param str logger: the logger to update the level. If not given, the root logger level is updated.
+        """Set the level of a logger for the duration of a test.
 
         .. versionchanged:: 3.4
-            The levels of the loggers changed by this function will be restored to their initial values at the
-            end of the test.
+            The levels of the loggers changed by this function will be
+            restored to their initial values at the end of the test.
+
+        :param int level: The level.
+        :param str logger: The logger to update. If not given, the root logger.
         """
         logger_obj = logging.getLogger(logger)
-        # save the original log-level to restore it during teardown
+        # Save the original log-level to restore it during teardown.
         self._initial_logger_levels.setdefault(logger, logger_obj.level)
         logger_obj.setLevel(level)
-        self._initial_handler_level = self.handler.level
+        if self._initial_handler_level is None:
+            self._initial_handler_level = self.handler.level
         self.handler.setLevel(level)
 
     @contextmanager
     def at_level(
         self, level: int, logger: Optional[str] = None
     ) -> Generator[None, None, None]:
-        """Context manager that sets the level for capturing of logs. After the end of the 'with' statement the
-        level is restored to its original value.
+        """Context manager that sets the level for capturing of logs. After
+        the end of the 'with' statement the level is restored to its original
+        value.
 
-        :param int level: the logger to level.
-        :param str logger: the logger to update the level. If not given, the root logger level is updated.
+        :param int level: The level.
+        :param str logger: The logger to update. If not given, the root logger.
         """
         logger_obj = logging.getLogger(logger)
         orig_level = logger_obj.level
@@ -462,7 +472,7 @@ class LogCaptureFixture:
             self.handler.setLevel(handler_orig_level)
 
 
-@pytest.fixture
+@fixture
 def caplog(request: FixtureRequest) -> Generator[LogCaptureFixture, None, None]:
     """Access and control log capturing.
 
@@ -474,7 +484,7 @@ def caplog(request: FixtureRequest) -> Generator[LogCaptureFixture, None, None]:
     * caplog.record_tuples   -> list of (logger_name, level, message) tuples
     * caplog.clear()         -> clear captured records and formatted log output string
     """
-    result = LogCaptureFixture(request.node)
+    result = LogCaptureFixture(request.node, _ispytest=True)
     yield result
     result._finalize()
 
@@ -495,7 +505,7 @@ def get_log_level_for_setting(config: Config, *setting_names: str) -> Optional[i
         return int(getattr(logging, log_level, log_level))
     except ValueError as e:
         # Python logging does not recognise this as a logging level
-        raise pytest.UsageError(
+        raise UsageError(
             "'{}' is not recognized as a logging level name for "
             "'{}'. Please consider passing the "
             "logging level num instead.".format(log_level, setting_name)
@@ -503,17 +513,16 @@ def get_log_level_for_setting(config: Config, *setting_names: str) -> Optional[i
 
 
 # run after terminalreporter/capturemanager are configured
-@pytest.hookimpl(trylast=True)
+@hookimpl(trylast=True)
 def pytest_configure(config: Config) -> None:
     config.pluginmanager.register(LoggingPlugin(config), "logging-plugin")
 
 
 class LoggingPlugin:
-    """Attaches to the logging module and captures log messages for each test.
-    """
+    """Attaches to the logging module and captures log messages for each test."""
 
     def __init__(self, config: Config) -> None:
-        """Creates a new plugin to capture log messages.
+        """Create a new plugin to capture log messages.
 
         The formatter can be safely shared across all handlers so
         create a single one for the entire test session here.
@@ -559,9 +568,9 @@ class LoggingPlugin:
             terminal_reporter = config.pluginmanager.get_plugin("terminalreporter")
             capture_manager = config.pluginmanager.get_plugin("capturemanager")
             # if capturemanager plugin is disabled, live logging still works.
-            self.log_cli_handler = _LiveLoggingStreamHandler(
-                terminal_reporter, capture_manager
-            )  # type: Union[_LiveLoggingStreamHandler, _LiveLoggingNullHandler]
+            self.log_cli_handler: Union[
+                _LiveLoggingStreamHandler, _LiveLoggingNullHandler
+            ] = _LiveLoggingStreamHandler(terminal_reporter, capture_manager)
         else:
             self.log_cli_handler = _LiveLoggingNullHandler()
         log_cli_formatter = self._create_formatter(
@@ -572,14 +581,14 @@ class LoggingPlugin:
         self.log_cli_handler.setFormatter(log_cli_formatter)
 
     def _create_formatter(self, log_format, log_date_format, auto_indent):
-        # color option doesn't exist if terminal plugin is disabled
+        # Color option doesn't exist if terminal plugin is disabled.
         color = getattr(self._config.option, "color", "no")
         if color != "no" and ColoredLevelFormatter.LEVELNAME_FMT_REGEX.search(
             log_format
         ):
-            formatter = ColoredLevelFormatter(
+            formatter: logging.Formatter = ColoredLevelFormatter(
                 create_terminal_writer(self._config), log_format, log_date_format
-            )  # type: logging.Formatter
+            )
         else:
             formatter = logging.Formatter(log_format, log_date_format)
 
@@ -590,17 +599,17 @@ class LoggingPlugin:
         return formatter
 
     def set_log_path(self, fname: str) -> None:
-        """Public method, which can set filename parameter for
-        Logging.FileHandler(). Also creates parent directory if
-        it does not exist.
+        """Set the filename parameter for Logging.FileHandler().
+
+        Creates parent directory if it does not exist.
 
         .. warning::
-            Please considered as an experimental API.
+            This is an experimental API.
         """
         fpath = Path(fname)
 
         if not fpath.is_absolute():
-            fpath = Path(str(self._config.rootdir), fpath)
+            fpath = self._config.rootpath / fpath
 
         if not fpath.parent.exists():
             fpath.parent.mkdir(exist_ok=True, parents=True)
@@ -634,7 +643,7 @@ class LoggingPlugin:
 
         return True
 
-    @pytest.hookimpl(hookwrapper=True, tryfirst=True)
+    @hookimpl(hookwrapper=True, tryfirst=True)
     def pytest_sessionstart(self) -> Generator[None, None, None]:
         self.log_cli_handler.set_when("sessionstart")
 
@@ -642,7 +651,7 @@ class LoggingPlugin:
             with catching_logs(self.log_file_handler, level=self.log_file_level):
                 yield
 
-    @pytest.hookimpl(hookwrapper=True, tryfirst=True)
+    @hookimpl(hookwrapper=True, tryfirst=True)
     def pytest_collection(self) -> Generator[None, None, None]:
         self.log_cli_handler.set_when("collection")
 
@@ -650,33 +659,31 @@ class LoggingPlugin:
             with catching_logs(self.log_file_handler, level=self.log_file_level):
                 yield
 
-    @pytest.hookimpl(hookwrapper=True)
+    @hookimpl(hookwrapper=True)
     def pytest_runtestloop(self, session: Session) -> Generator[None, None, None]:
-        """Runs all collected test items."""
-
         if session.config.option.collectonly:
             yield
             return
 
         if self._log_cli_enabled() and self._config.getoption("verbose") < 1:
-            # setting verbose flag is needed to avoid messy test progress output
+            # The verbose flag is needed to avoid messy test progress output.
             self._config.option.verbose = 1
 
         with catching_logs(self.log_cli_handler, level=self.log_cli_level):
             with catching_logs(self.log_file_handler, level=self.log_file_level):
-                yield  # run all the tests
+                yield  # Run all the tests.
 
-    @pytest.hookimpl
+    @hookimpl
     def pytest_runtest_logstart(self) -> None:
         self.log_cli_handler.reset()
         self.log_cli_handler.set_when("start")
 
-    @pytest.hookimpl
+    @hookimpl
     def pytest_runtest_logreport(self) -> None:
         self.log_cli_handler.set_when("logreport")
 
     def _runtest_for(self, item: nodes.Item, when: str) -> Generator[None, None, None]:
-        """Implements the internals of pytest_runtest_xxx() hook."""
+        """Implement the internals of the pytest_runtest_xxx() hooks."""
         with catching_logs(
             self.caplog_handler, level=self.log_level,
         ) as caplog_handler, catching_logs(
@@ -692,21 +699,21 @@ class LoggingPlugin:
             log = report_handler.stream.getvalue().strip()
             item.add_report_section(when, "log", log)
 
-    @pytest.hookimpl(hookwrapper=True)
+    @hookimpl(hookwrapper=True)
     def pytest_runtest_setup(self, item: nodes.Item) -> Generator[None, None, None]:
         self.log_cli_handler.set_when("setup")
 
-        empty = {}  # type: Dict[str, List[logging.LogRecord]]
+        empty: Dict[str, List[logging.LogRecord]] = {}
         item._store[caplog_records_key] = empty
         yield from self._runtest_for(item, "setup")
 
-    @pytest.hookimpl(hookwrapper=True)
+    @hookimpl(hookwrapper=True)
     def pytest_runtest_call(self, item: nodes.Item) -> Generator[None, None, None]:
         self.log_cli_handler.set_when("call")
 
         yield from self._runtest_for(item, "call")
 
-    @pytest.hookimpl(hookwrapper=True)
+    @hookimpl(hookwrapper=True)
     def pytest_runtest_teardown(self, item: nodes.Item) -> Generator[None, None, None]:
         self.log_cli_handler.set_when("teardown")
 
@@ -714,11 +721,11 @@ class LoggingPlugin:
         del item._store[caplog_records_key]
         del item._store[caplog_handler_key]
 
-    @pytest.hookimpl
+    @hookimpl
     def pytest_runtest_logfinish(self) -> None:
         self.log_cli_handler.set_when("finish")
 
-    @pytest.hookimpl(hookwrapper=True, tryfirst=True)
+    @hookimpl(hookwrapper=True, tryfirst=True)
     def pytest_sessionfinish(self) -> Generator[None, None, None]:
         self.log_cli_handler.set_when("sessionfinish")
 
@@ -726,7 +733,7 @@ class LoggingPlugin:
             with catching_logs(self.log_file_handler, level=self.log_file_level):
                 yield
 
-    @pytest.hookimpl
+    @hookimpl
     def pytest_unconfigure(self) -> None:
         # Close the FileHandler explicitly.
         # (logging.shutdown might have lost the weakref?!)
@@ -734,9 +741,7 @@ class LoggingPlugin:
 
 
 class _FileHandler(logging.FileHandler):
-    """
-    Custom FileHandler with pytest tweaks.
-    """
+    """A logging FileHandler with pytest tweaks."""
 
     def handleError(self, record: logging.LogRecord) -> None:
         # Handled by LogCaptureHandler.
@@ -744,27 +749,23 @@ class _FileHandler(logging.FileHandler):
 
 
 class _LiveLoggingStreamHandler(logging.StreamHandler):
-    """
-    Custom StreamHandler used by the live logging feature: it will write a newline before the first log message
-    in each test.
+    """A logging StreamHandler used by the live logging feature: it will
+    write a newline before the first log message in each test.
 
-    During live logging we must also explicitly disable stdout/stderr capturing otherwise it will get captured
-    and won't appear in the terminal.
+    During live logging we must also explicitly disable stdout/stderr
+    capturing otherwise it will get captured and won't appear in the
+    terminal.
     """
 
     # Officially stream needs to be a IO[str], but TerminalReporter
     # isn't. So force it.
-    stream = None  # type: TerminalReporter # type: ignore
+    stream: TerminalReporter = None  # type: ignore
 
     def __init__(
         self,
         terminal_reporter: TerminalReporter,
         capture_manager: Optional[CaptureManager],
     ) -> None:
-        """
-        :param _pytest.terminal.TerminalReporter terminal_reporter:
-        :param _pytest.capture.CaptureManager capture_manager:
-        """
         logging.StreamHandler.__init__(self, stream=terminal_reporter)  # type: ignore[arg-type]
         self.capture_manager = capture_manager
         self.reset()
@@ -772,11 +773,11 @@ class _LiveLoggingStreamHandler(logging.StreamHandler):
         self._test_outcome_written = False
 
     def reset(self) -> None:
-        """Reset the handler; should be called before the start of each test"""
+        """Reset the handler; should be called before the start of each test."""
         self._first_record_emitted = False
 
     def set_when(self, when: Optional[str]) -> None:
-        """Prepares for the given test phase (setup/call/teardown)"""
+        """Prepare for the given test phase (setup/call/teardown)."""
         self._when = when
         self._section_name_shown = False
         if when == "start":
@@ -807,7 +808,7 @@ class _LiveLoggingStreamHandler(logging.StreamHandler):
 
 
 class _LiveLoggingNullHandler(logging.NullHandler):
-    """A handler used when live logging is disabled."""
+    """A logging handler used when live logging is disabled."""
 
     def reset(self) -> None:
         pass
